from commmons import html_from_url, get_host_url


def scrape(url):
    tree = html_from_url(url)
    host_url = get_host_url(url)

    divs = tree.xpath("//div[contains(@class, 'video-item')]")
    for div in divs:
        if "data-id" not in div.attrib:
            continue

        dataid = div.attrib["data-id"]
        atags = div.xpath("./a")

        if not atags:
            continue

        imgs = div.xpath("./a/picture/img")
        if not imgs:
            continue

        atag = atags[0]
        img = imgs[0]

        yield {
            "fileid": "sb" + dataid,
            "sourceurl": host_url + atag.attrib["href"],
            "filename": img.attrib["alt"],
            "thumbnailurl": img.attrib["data-src"]
        }



from typing import Iterator

from commmons import md5
from yt_dlp import YoutubeDL


def scrape_ytdl(url: str) -> Iterator[dict]:
    ydl = YoutubeDL({'outtmpl': '%(id)s.%(ext)s'})

    with ydl:
        result = ydl.extract_info(url, download=False)

    if 'entries' in result:
        # Can be a playlist or a list of videos
        videos = result['entries']
    else:
        # Just a video
        videos = [result]

    for v in videos:
        yield {
            "fileid": md5(v["extractor"]) + v["id"],
            "filename": v["title"],
            "sourceurl": v["original_url"],
            "thumbnailurl": v["thumbnail"]
        }



import requests
from commmons import html_from_url
from bs4 import BeautifulSoup


def html_from_url_with_headers(url, referer=None):
    return html_from_url(url, headers={
        "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,*/*;q=0.8",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:90.0) Gecko/20100101 Firefox/90.0",
        "Referer": referer or url
    })


def html_from_url_with_bs4(url, **kwargs):
    r = requests.get(
        url,
        headers={
            "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,*/*;q=0.8",
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:90.0) Gecko/20100101 Firefox/90.0",
            "Referer": url,
        },
        **kwargs
    )
    return BeautifulSoup(r.text, "lxml")



import itertools
import sys
from dataclasses import dataclass, field
from typing import Iterator, Callable


@dataclass
class Watchlist:
    url: str
    max_items: int = field(default=None)
    page_min: int = field(default=1)
    page_max: int = field(default=1)

    @property
    def is_multipage(self):
        return "${page}" in self.url

    def __post_init__(self):
        assert self.url
        if self.is_multipage:
            assert isinstance(self.page_min, int)
            assert isinstance(self.page_max, int)
            assert self.page_min < self.page_max

    def scrape(self, scraper: Callable[[str], Iterator[dict]], url=None):
        def generate():
            for page in range(self.page_min, self.page_max + 1):
                substituted_url = (url or self.url).replace("${page}", str(page))
                yield from scraper(substituted_url)

        yield from itertools.islice(generate(), self.max_items or sys.maxsize)



from typing import Iterator

from commmons import html_from_url, head, md5, last

PREFIX = md5("HellPorno")


def scrape_div(div):
    atag = head(div.xpath(".//a"))
    if atag is None:
        return

    imgtag = head(div.xpath(".//img"))
    if imgtag is None:
        return

    span = head(div.xpath(".//*[@class='video-title']"))
    if span is None:
        return

    sourceurl: str = atag.attrib["href"]
    fileid = PREFIX + last([s for s in sourceurl.split("/") if s])

    yield dict(
        fileid=fileid,
        filename=span.text,
        sourceurl=sourceurl,
        thumbnailurl=imgtag.attrib["src"]
    )


def scrape(url: str) -> Iterator[dict]:
    root = html_from_url(url, headers={
        "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,*/*;q=0.8",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:90.0) Gecko/20100101 Firefox/90.0"
    })

    divs = root.xpath("//div[@class='thumb']")

    for div in divs:
        yield from scrape_div(div)
